----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:14:48 05/30/2011 
-- Design Name: 
-- Module Name:    modm_counter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: Counts to M-1, when a max tick signal is asserted, and the counter is reset to begin again.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity modm_counter is
	 Generic (	
					N : integer := 4; -- no bits in counter
					M : integer := 10 -- max value
				);
    Port 	( 		
					clk, reset : in  STD_LOGIC;
					max_tick : out  STD_LOGIC;
					q : out  STD_LOGIC_VECTOR (N-1 downto 0)
				);
end modm_counter;

architecture Behavioral of modm_counter is
signal c_reg, c_next : unsigned(N-1 downto 0) := (others=>'0');
signal i_max_tick : std_logic;
begin

	process (clk)
		begin
			if (clk'event and clk='1') then 
				c_reg <= c_next;
			end if;
	end process;
		
	c_next <= (others=>'0') when (i_max_tick='1') OR (reset ='1') else c_reg + 1;

	i_max_tick <= '1' when c_reg = (M-1) else '0';

	max_tick <= i_max_tick;
	
	q <= std_logic_vector(c_reg);

end Behavioral;

